print("A.6: This script loads the SDP solution together with the 'ineq_codeg', 'ineq_tightpath' and 'flagmatic_flags-pruned.rat' files, and then formally verifies correctness of the identity (2.2) in Lemma 2.8.\n")

def load_vector(filename, field=QQ):
	V = []
	f = open(filename)
	for line in f: V.append(field(line[:-1]))
	f.close()
	return V
# load_vector

def load_vector_list(filename, field=QQ):
	L = []
	f = open(filename)
	for line in f: L.append(vector(field,line[:-1].split()))
	f.close()
	return L
# load_vector_list

def proof_assert(expr, comment):
	if (not expr):
		print("There is a problem with the verification of the proof: Claim '%s' is FALSE" % comment)
		sys.exit(1)
# proof_assert

print("Loading the coefficients of the 905 CODEGREE-BOUND expressions D")
ineq_codeg = load_vector_list('./ineq_codeg')
print("Loading the postive rational values u_D")
u_rat = load_vector("./solution-u")
proof_assert(min(u_rat)>0 , "positivity of u")

print("Loading the coefficients of the 3 TIGHT-PATH expressions P")
ineq_tightpath = load_vector_list('./ineq_tightpath')
print("Loading the positive rational values c_0, c_1 and c_2")
c_rat = load_vector("./solution-c")
proof_assert(min(c_rat)>0 , "positivity of c") # thus we can use the complementary slackness principle on short tight-paths

IQI={}
iotas_flagmatic = [7,8,9,10,11,14] # index of the type iota in flagmatic-flags.rat (from flagmatic 1.5.1)
for iota in [1,2,3,4,5,6]:
	print("Loading the matrices I_%d and Q_%d for the POSITIVE-SEMIDEFINITE expression" %(iota,iota))
	(I,QblockA,QblockB)=map(lambda x:matrix(load_vector_list(x%iota)), ("./solution-I_%d","./solution-Q_%d.blockA","./solution-Q_%d.blockB"))
	proof_assert(QblockA.is_positive_definite() and QblockB.is_positive_definite(), "positive-definiteness of Q_%d"%iota)
	IQI[iotas_flagmatic[iota-1]] = I.T*block_diagonal_matrix(QblockA,QblockB)*I
#for iota

print("\nComputing the non-negative vector on the RHS")
RHS = matrix(ineq_codeg).T * vector(u_rat) + matrix(ineq_tightpath).T * vector(c_rat)
f=open('./flagmatic_flags-pruned.rat') #pair densities generated by flagmatic 1.5
for line in f:
	(coord,iota,a,b,num,denom) = map(int,line.split())
	if (iota in iotas_flagmatic): RHS[coord-1] += IQI[iota][a-1,b-1] * (1+(a!=b)) * num/denom # note that flagmatic is indexing vectors/matrices from 1, we do from 0
f.close()

# the numbers of 7-vertex K4- graphs with a given number of edges (note that the flagmatic list is sorted wrt number of edges)
F7_edgestat = [1, 1, 3, 9, 32, 102, 304, 752, 1451, 2022, 1909, 1118, 374, 70, 8, 1]
LHS = [] #LHS is N - 3*E = 1 - 4*E
for i in range(16): LHS += F7_edgestat[i] * [1-4*i/35]

slack = vector(LHS) - RHS
proof_assert(min(slack)==0, "non-negativity of slack variables")
slack_coordinates_num = len(list(filter(lambda x:x>0,slack)))
proof_assert(slack_coordinates_num == 7910, "#slack variables == |G_7|") # note that coordinates corresponding to E_T will not have any slack by complementary-slackness, and |E_T \cap F_7| == 247 == 8157 - 7910

print("Success! We have coordinate-wise verified that LHS >= RHS.\nIn particular, any feasible 3-graph must have density <= 1/4, and complementary slackness yields all the claimed properties.")
